@@ -75,6 +75,22 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
 
     const QgsFeature &feature() { return mFeature; }
 
+    /**
+     * Returns the feature that is currently displayed in the form with all
+     * the changes received on editing the values in the widgets.
+     *
+     * \since QGIS 3.16
+     */
+    QgsFeature currentFormFeature() const { return mCurrentFormFeature; }
+
+    /**
+     * Displays a warning message in the form message bar
+     * \param message message string
+     * \see mode()
+     * \since QGIS 3.12
+     */
+    void displayWarning( const QString &message );
+
     // TODO QGIS 4.0 - make private
 
     /**
@@ -175,6 +191,14 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     QString aggregateFilter() const;
 
+    /**
+     * Sets an additional expression context scope to be used
+     * for calculations in this form.
+     *
+     * \since QGIS 3.16
+     */
+    void setExtraContextScope( QgsExpressionContextScope *extraScope SIP_TRANSFER );
+
   signals:
 
     /**
@@ -269,6 +293,17 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     bool save();
 
+    /**
+     * Save all the values from the editors to the layer.
+     *
+     * \param error if specified, will be set to an explanatory error message if an error occurs while saving the form.
+     *
+     * \returns TRUE if save was successful
+     *
+     * \since QGIS 3.18
+     */
+    bool saveWithDetails( QString *error SIP_OUT = nullptr );
+
     /**
      * Sets all values to the values of the current feature
      */
@@ -285,6 +320,18 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     void refreshFeature();
 
+    /**
+     * Is called in embedded forms when an \a attribute value in the parent form
+     * has changed to \a newValue.
+     *
+     * Notify the form widgets that something has changed in case they
+     * have filter expressions that depend on the parent form scope.
+     *
+     * \since QGIS 3.14
+     */
+    void parentFormValueChanged( const QString &attribute, const QVariant &newValue );
+
+
   private slots:
     void onAttributeChanged( const QVariant &value, const QVariantList &additionalFieldValues );
     void onAttributeAdded( int idx );
@@ -325,6 +372,8 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
 
     bool fieldIsEditable( const QgsVectorLayer &layer, int fieldIndex, QgsFeatureId fid ) const;
 
+    void updateDefaultValueDependencies();
+
     struct WidgetInfo
     {
       QWidget *widget = nullptr;
@@ -350,7 +399,7 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
     void scanForEqualAttributes( QgsFeatureIterator &fit, QSet< int > &mixedValueFields, QHash< int, QVariant > &fieldSharedValues ) const;
 
     //! Save single feature or add feature edits
-    bool saveEdits();
+    bool saveEdits( QString *error );
 
     //! fill up dependency map for default values
     void createDefaultValueDependencies();
@@ -358,39 +407,44 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
     //! update the default values in the fields after a referenced field changed
     bool updateDefaultValues( const int originIdx );
 
-    int messageTimeout();
     void clearMultiEditMessages();
     void pushSelectedFeaturesMessage();
     void runSearchSelect( QgsVectorLayer::SelectBehavior behavior );
 
     QString createFilterExpression() const;
 
+    QgsExpressionContext createExpressionContext( const QgsFeature &feature ) const;
+
     //! constraints management
     void updateAllConstraints();
     void updateConstraints( QgsEditorWidgetWrapper *w );
     void updateContainersVisibility();
     void updateConstraint( const QgsFeature &ft, QgsEditorWidgetWrapper *eww );
-    bool currentFormFeature( QgsFeature &feature );
+    void updateLabels();
+    bool currentFormValuesFeature( QgsFeature &feature );
     bool currentFormValidConstraints( QStringList &invalidFields, QStringList &descriptions );
     QList<QgsEditorWidgetWrapper *> constraintDependencies( QgsEditorWidgetWrapper *w );
 
-    QgsRelationWidgetWrapper *setupRelationWidgetWrapper( const QgsRelation &rel, const QgsAttributeEditorContext &context );
+    Q_DECL_DEPRECATED QgsRelationWidgetWrapper *setupRelationWidgetWrapper( const QgsRelation &rel, const QgsAttributeEditorContext &context ) SIP_DEPRECATED;
+    QgsRelationWidgetWrapper *setupRelationWidgetWrapper( const QString &relationWidgetTypeId, const QgsRelation &rel, const QgsAttributeEditorContext &context );
 
     QgsVectorLayer *mLayer = nullptr;
     QgsFeature mFeature;
+    QgsFeature mCurrentFormFeature;
     QgsMessageBar *mMessageBar = nullptr;
     bool mOwnsMessageBar;
     QgsMessageBarItem *mMultiEditUnsavedMessageBarItem = nullptr;
     QgsMessageBarItem *mMultiEditMessageBarItem = nullptr;
     QList<QgsWidgetWrapper *> mWidgets;
     QgsAttributeEditorContext mContext;
+    std::unique_ptr<QgsExpressionContextScope> mExtraContextScope;
     QDialogButtonBox *mButtonBox = nullptr;
     QWidget *mSearchButtonBox = nullptr;
     QList<QgsAttributeFormInterface *> mInterfaces;
     QMap< int, QgsAttributeFormEditorWidget * > mFormEditorWidgets;
     QList< QgsAttributeFormWidget *> mFormWidgets;
-    QgsExpressionContext mExpressionContext;
     QMap<const QgsVectorLayerJoinInfo *, QgsFeature> mJoinedFeatures;
+    QMap<QLabel *, QgsProperty> mLabelDataDefinedProperties;
     bool mValuesInitialized = false;
     bool mDirty = false;
     bool mIsSettingFeature = false;
