@@ -37,7 +37,7 @@ class QgsFieldFormatter;
 
 /**
  * \ingroup gui
- * A model backed by a QgsVectorLayerCache which is able to provide
+ * \brief A model backed by a QgsVectorLayerCache which is able to provide
  * feature/attribute information to a QAbstractItemView.
  *
  * \brief
@@ -57,7 +57,7 @@ class GUI_EXPORT QgsAttributeTableModel: public QAbstractTableModel
       FieldIndexRole,               //!< Get the field index of this column
       UserRole,                     //!< Start further roles starting from this role
       // Insert new values here, SortRole needs to be the last one
-      SortRole,                     //!< Roles used for sorting start here
+      SortRole,                     //!< Role used for sorting start here
     };
 
   public:
@@ -221,10 +221,11 @@ class GUI_EXPORT QgsAttributeTableModel: public QAbstractTableModel
      */
     void setRequest( const QgsFeatureRequest &request );
 
+    // TODO QGIS 4: return copy instead of reference
+
     /**
      * Gets the the feature request
      */
-    // TODO QGIS 3: return copy instead of reference
     const QgsFeatureRequest &request() const;
 
     /**
@@ -317,10 +318,8 @@ class GUI_EXPORT QgsAttributeTableModel: public QAbstractTableModel
     /**
      * Launched when a feature has been added
      * \param fid feature id
-     * \param resettingModel set to true if model is in the process of being reset
-     * and the normal begin/EndInsertRows calls should not be made
      */
-    virtual void featureAdded( QgsFeatureId fid, bool resettingModel = false );
+    virtual void featureAdded( QgsFeatureId fid );
 
     /**
      * Launched when layer has been deleted
@@ -361,8 +360,6 @@ class GUI_EXPORT QgsAttributeTableModel: public QAbstractTableModel
      */
     virtual bool loadFeatureAtId( QgsFeatureId fid ) const;
 
-    bool fieldIsEditable( const QgsVectorLayer &layer, int fieldIndex, QgsFeatureId fid ) const;
-
     QgsFeatureRequest mFeatureRequest;
 
     struct SortCache
@@ -385,6 +382,9 @@ class GUI_EXPORT QgsAttributeTableModel: public QAbstractTableModel
     //! Flag for massive changes operations, set by edit command or rollback
     bool mBulkEditCommandRunning = false;
 
+    //! TRUE if model is in the midst of a reset operation
+    bool mResettingModel = false;
+
     //! Sets the flag for massive changes operations
     void bulkEditCommandStarted();
 
