@@ -25,6 +25,9 @@
 #include "qgsattributeform.h"
 #include "qgis_gui.h"
 
+#include <QPointer>
+#include <QUuid>
+
 class QgsFeatureRequest;
 class QgsMapLayerAction;
 class QgsScrollArea;
@@ -32,7 +35,7 @@ class QgsFieldConditionalFormatWidget;
 
 /**
  * \ingroup gui
- * This widget is used to show the attributes of a set of features of a QgsVectorLayer.
+ * \brief This widget is used to show the attributes of a set of features of a QgsVectorLayer.
  * The attributes can be edited.
  * It supports two different layouts: the table layout, in which the attributes for the features
  * are shown in a table and the editor layout, where the features are shown as a selectable list
