@@ -32,18 +32,21 @@
 #include "qgsexpressioncontext.h"
 #include "qgsmaplayer.h"
 #include "qgsgeometry.h"
+#include "qgstemporalrangeobject.h"
+#include "qgsmapclippingregion.h"
 
 class QPainter;
 
 class QgsCoordinateTransform;
 class QgsScaleCalculator;
 class QgsMapRendererJob;
+class QgsRenderedFeatureHandlerInterface;
 
 /**
  * \class QgsLabelBlockingRegion
  * \ingroup core
  *
- * Label blocking region (in map coordinates and CRS).
+ * \brief Label blocking region (in map coordinates and CRS).
  *
  * \since QGIS 3.6
 */
@@ -66,7 +69,7 @@ class CORE_EXPORT QgsLabelBlockingRegion
 
 /**
  * \ingroup core
- * The QgsMapSettings class contains configuration for rendering of the map.
+ * \brief The QgsMapSettings class contains configuration for rendering of the map.
  * The rendering itself is done by QgsMapRendererJob subclasses.
  *
  * In order to set up QgsMapSettings instance, it is necessary to set at least
@@ -81,50 +84,92 @@ class CORE_EXPORT QgsLabelBlockingRegion
  *
  * \since QGIS 2.4
  */
-class CORE_EXPORT QgsMapSettings
+class CORE_EXPORT QgsMapSettings : public QgsTemporalRangeObject
 {
   public:
     QgsMapSettings();
 
     /**
      * Returns geographical coordinates of the rectangle that should be rendered.
-     * The actual visible extent used for rendering could be slightly different
-     * since the given extent may be expanded in order to fit the aspect ratio
-     * of output size. Use visibleExtent() to get the resulting extent.
+     *
+     * \warning The actual visible extent used for rendering can be significantly different from this
+     * value, since the given extent may be expanded in order to fit the aspect ratio
+     * of the outputSize(). Use visibleExtent() to get the actual extent which will be rendered.
+     *
+     * \see visibleExtent()
+     * \see setExtent()
      */
     QgsRectangle extent() const;
 
     /**
-     * Set coordinates of the rectangle which should be rendered.
-     * The actual visible extent used for rendering could be slightly different
-     * since the given extent may be expanded in order to fit the aspect ratio
-     * of output size. Use visibleExtent() to get the resulting extent.
+     * Sets the coordinates of the rectangle which should be rendered.
+     *
+     * \warning The actual visible extent used for rendering can be significantly different
+     * from the specified extent, since the given extent may be expanded in order to match the
+     * aspect ratio of outputSize(). Use visibleExtent() to retrieve the actual extent to be rendered.
+     *
+     * \see visibleExtent()
+     * \see extent()
      */
     void setExtent( const QgsRectangle &rect, bool magnified = true );
 
-    //! Returns the size of the resulting map image
+    /**
+     * Returns the buffer in map units to use around the visible extent for rendering
+     * symbols whose corresponding geometries are outside the visible extent.
+     * \see setExtentBuffer()
+     * \since QGIS 3.10
+     */
+    double extentBuffer() const;
+
+    /**
+     * Sets the buffer in map units to use around the visible extent for rendering
+     * symbols whose corresponding geometries are outside the visible extent. This
+     * is useful when using tiles to avoid cut symbols at tile boundaries.
+     * \since QGIS 3.10
+     */
+    void setExtentBuffer( double buffer );
+
+    /**
+     * Returns the size of the resulting map image, in pixels.
+     *
+     * \see deviceOutputSize()
+     * \see setOutputSize()
+     */
     QSize outputSize() const;
-    //! Sets the size of the resulting map image
+
+    /**
+     * Sets the \a size of the resulting map image, in pixels.
+     *
+     * \see outputSize()
+     */
     void setOutputSize( QSize size );
 
     /**
-     * Returns device pixel ratio
+     * Returns the device pixel ratio.
+     *
      * Common values are 1 for normal-dpi displays and 2 for high-dpi "retina" displays.
      * \since QGIS 3.4
      */
     float devicePixelRatio() const;
 
     /**
-     * Sets the device pixel ratio
+     * Sets the device pixel ratio.
+     *
      * Common values are 1 for normal-dpi displays and 2 for high-dpi "retina" displays.
      * \since QGIS 3.4
      */
     void setDevicePixelRatio( float dpr );
 
     /**
-     * Returns the device output size of the map canvas
+     * Returns the device output size of the map render.
+     *
      * This is equivalent to the output size multiplicated
      * by the device pixel ratio.
+     *
+     * \see outputSize()
+     * \see devicePixelRatio()
+     * \see setOutputSize()
+     *
      * \since QGIS 3.4
      */
     QSize deviceOutputSize() const;
@@ -144,20 +189,29 @@ class CORE_EXPORT QgsMapSettings
     void setRotation( double rotation );
 
     /**
-     * Returns DPI used for conversion between real world units (e.g. mm) and pixels
-     * Default value is 96
+     * Returns the DPI (dots per inch) used for conversion between real world units (e.g. mm) and pixels.
+     *
+     * The default value is 96 dpi.
+     *
+     * \see setOutputDpi()
      */
     double outputDpi() const;
-    //! Sets DPI used for conversion between real world units (e.g. mm) and pixels
+
+    /**
+     * Sets the \a dpi (dots per inch) used for conversion between real world units (e.g. mm) and pixels.
+     *
+     * \see outputDpi()
+     */
     void setOutputDpi( double dpi );
 
     /**
      * Set the magnification factor.
      * \param factor the factor of magnification
+     * \param center optional point to re-center the map
      * \see magnificationFactor()
      * \since QGIS 2.16
      */
-    void setMagnificationFactor( double factor );
+    void setMagnificationFactor( double factor, const QgsPointXY *center = nullptr );
 
     /**
      * Returns the magnification factor.
@@ -167,59 +221,113 @@ class CORE_EXPORT QgsMapSettings
     double magnificationFactor() const;
 
     /**
-     * Gets list of layer IDs for map rendering
-     * The layers are stored in the reverse order of how they are rendered (layer with index 0 will be on top)
+     * Returns the list of layer IDs which will be rendered in the map.
+     *
+     * The layers are stored in the reverse order of how they are rendered (layer with index 0 will be on top).
+     *
+     * \see layers()
+     * \see setLayers()
      */
     QStringList layerIds() const;
 
     /**
-     * Gets list of layers for map rendering
+     * Returns the list of layers which will be rendered in the map.
+     *
      * The layers are stored in the reverse order of how they are rendered (layer with index 0 will be on top)
+     *
+     * \see setLayers()
+     * \see layerIds()
      */
     QList<QgsMapLayer *> layers() const;
 
     /**
-     * Set list of layers for map rendering. The layers must be registered in QgsProject.
+     * Sets the list of \a layers to render in the map.
+     *
      * The layers are stored in the reverse order of how they are rendered (layer with index 0 will be on top)
      *
      * \note Any non-spatial layers will be automatically stripped from the list (since they cannot be rendered!).
+     *
+     * \see layers()
+     * \see layerIds()
      */
     void setLayers( const QList<QgsMapLayer *> &layers );
 
     /**
-     * Gets map of map layer style overrides (key: layer ID, value: style name) where a different style should be used instead of the current one
+     * Returns the map of map layer style overrides (key: layer ID, value: style name) where a different style should be used instead of the current one.
+     *
+     * \see setLayerStyleOverrides()
      * \since QGIS 2.8
      */
     QMap<QString, QString> layerStyleOverrides() const;
 
     /**
-     * Set map of map layer style overrides (key: layer ID, value: style name) where a different style should be used instead of the current one
+     * Sets the map of map layer style \a overrides (key: layer ID, value: style name) where a different style should be used instead of the current one.
+     *
+     * \see layerStyleOverrides()
      * \since QGIS 2.8
      */
     void setLayerStyleOverrides( const QMap<QString, QString> &overrides );
 
     /**
-     * Gets custom rendering flags. Layers might honour these to alter their rendering.
-     *  \returns custom flags strings, separated by ';'
+     * Returns custom rendering flags. Layers might honour these to alter their rendering.
+     * \returns custom flags strings, separated by ';'
      * \see setCustomRenderFlags()
      * \since QGIS 2.16
+     * \deprecated use \see customRenderingFlags().
      */
-    QString customRenderFlags() const { return mCustomRenderFlags; }
+    Q_DECL_DEPRECATED QString customRenderFlags() const { return mCustomRenderFlags; }
 
     /**
      * Sets the custom rendering flags. Layers might honour these to alter their rendering.
      * \param customRenderFlags custom flags strings, separated by ';'
      * \see customRenderFlags()
      * \since QGIS 2.16
+     * \deprecated use \see setCustomRenderingFlag() instead.
      */
-    void setCustomRenderFlags( const QString &customRenderFlags ) { mCustomRenderFlags = customRenderFlags; }
+    Q_DECL_DEPRECATED void setCustomRenderFlags( const QString &customRenderFlags ) { mCustomRenderFlags = customRenderFlags; }
 
-    //! sets destination coordinate reference system
+    /**
+     * Returns any custom rendering flags. Layers might honour these to alter their rendering.
+     * \returns a map of custom flags
+     * \see setCustomRenderingFlag()
+     * \since QGIS 3.12
+     */
+    QVariantMap customRenderingFlags() const { return mCustomRenderingFlags; }
+
+    /**
+     * Sets a custom rendering \a flag. Layers might honour these to alter their rendering.
+     * \param flag the flag name
+     * \param value the flag value
+     * \see customRenderingFlags()
+     * \since QGIS 3.12
+     */
+    void setCustomRenderingFlag( const QString &flag, const QVariant &value ) { mCustomRenderingFlags[flag] = value; }
+
+    /**
+     * Clears the specified custom rendering \a flag.
+     * \param flag the flag name
+     * \see setCustomRenderingFlag()
+     * \since QGIS 3.12
+     */
+    void clearCustomRenderingFlag( const QString &flag ) { mCustomRenderingFlags.remove( flag ); }
+
+    /**
+     * Sets the destination \a crs (coordinate reference system) for the map render.
+     *
+     * \see destinationCrs()
+     */
     void setDestinationCrs( const QgsCoordinateReferenceSystem &crs );
-    //! returns CRS of destination coordinate reference system
+
+    /**
+     * Returns the destination coordinate reference system for the map render.
+     *
+     * \see setDestinationCrs()
+     */
     QgsCoordinateReferenceSystem destinationCrs() const;
 
-    //! Gets units of map's geographical coordinates - used for scale calculation
+    /**
+     * Returns the units of the map's geographical coordinates - used for scale calculation.
+     */
     QgsUnitTypes::DistanceUnit mapUnits() const;
 
     /**
@@ -240,14 +348,32 @@ class CORE_EXPORT QgsMapSettings
      */
     QString ellipsoid() const { return mEllipsoid; }
 
-    //! Sets the background color of the map
+    /**
+     * Sets the background \a color of the map.
+     *
+     * \see backgroundColor()
+     */
     void setBackgroundColor( const QColor &color ) { mBackgroundColor = color; }
-    //! Gets the background color of the map
+
+    /**
+     * Returns the background color of the map.
+     *
+     * \see setBackgroundColor()
+     */
     QColor backgroundColor() const { return mBackgroundColor; }
 
-    //! Sets color that is used for drawing of selected vector features
+    /**
+     * Sets the \a color that is used for drawing of selected vector features.
+     *
+     * \see selectionColor()
+     */
     void setSelectionColor( const QColor &color ) { mSelectionColor = color; }
-    //! Gets color that is used for drawing of selected vector features
+
+    /**
+     * Returns the color that is used for drawing of selected vector features.
+     *
+     * \see setSelectionColor()
+     */
     QColor selectionColor() const { return mSelectionColor; }
 
     //! Enumeration of flags that adjust the way the map is rendered
@@ -264,6 +390,9 @@ class CORE_EXPORT QgsMapSettings
       RenderMapTile            = 0x100, //!< Draw map such that there are no problems between adjacent tiles
       RenderPartialOutput      = 0x200, //!< Whether to make extra effort to update map image with partially rendered layers (better for interactive map canvas). Added in QGIS 3.0
       RenderPreviewJob         = 0x400, //!< Render is a 'canvas preview' render, and shortcuts should be taken to ensure fast rendering
+      RenderBlocking           = 0x800, //!< Render and load remote sources in the same thread to ensure rendering remote sources (svg and images). WARNING: this flag must NEVER be used from GUI based applications (like the main QGIS application) or crashes will result. Only for use in external scripts or QGIS server.
+      LosslessImageRendering   = 0x1000, //!< Render images losslessly whenever possible, instead of the default lossy jpeg rendering used for some destination devices (e.g. PDF). This flag only works with builds based on Qt 5.13 or later.
+      Render3DMap              = 0x2000, //!< Render is for a 3D map
       // TODO: ignore scale-based visibility (overview)
     };
     Q_DECLARE_FLAGS( Flags, Flag )
@@ -413,6 +542,13 @@ class CORE_EXPORT QgsMapSettings
      */
     QgsPointXY layerToMapCoordinates( const QgsMapLayer *layer, QgsPointXY point ) const;
 
+    /**
+     * \brief transform point coordinates from layer's CRS to output CRS
+     * \returns the transformed point
+     * \since QGIS 3.16
+     */
+    QgsPoint layerToMapCoordinates( const QgsMapLayer *layer, const QgsPoint &point ) const;
+
     /**
      * \brief transform rectangle from layer's CRS to output CRS
      * \see layerExtentToOutputExtent() if you want to transform a bounding box
@@ -426,6 +562,13 @@ class CORE_EXPORT QgsMapSettings
      */
     QgsPointXY mapToLayerCoordinates( const QgsMapLayer *layer, QgsPointXY point ) const;
 
+    /**
+     * \brief transform point coordinates from output CRS to layer's CRS
+     * \returns the transformed point
+     * \since QGIS 3.16
+     */
+    QgsPoint mapToLayerCoordinates( const QgsMapLayer *layer, const QgsPoint &point ) const;
+
     /**
      * \brief transform rectangle from output CRS to layer's CRS
      * \see outputExtentToLayerExtent() if you want to transform a bounding box
@@ -450,14 +593,16 @@ class CORE_EXPORT QgsMapSettings
 
     /**
      * Sets the segmentation tolerance applied when rendering curved geometries
-    \param tolerance the segmentation tolerance*/
+     * \param tolerance the segmentation tolerance
+    */
     void setSegmentationTolerance( double tolerance ) { mSegmentationTolerance = tolerance; }
     //! Gets the segmentation tolerance applied when rendering curved geometries
     double segmentationTolerance() const { return mSegmentationTolerance; }
 
     /**
      * Sets segmentation tolerance type (maximum angle or maximum difference between curve and approximation)
-    \param type the segmentation tolerance typename*/
+     * \param type the segmentation tolerance typename
+    */
     void setSegmentationToleranceType( QgsAbstractGeometry::SegmentationToleranceType type ) { mSegmentationToleranceType = type; }
     //! Gets segmentation tolerance type (maximum angle or maximum difference between curve and approximation)
     QgsAbstractGeometry::SegmentationToleranceType segmentationToleranceType() const { return mSegmentationToleranceType; }
@@ -531,6 +676,101 @@ class CORE_EXPORT QgsMapSettings
      */
     QList< QgsLabelBlockingRegion > labelBlockingRegions() const { return mLabelBlockingRegions; }
 
+    /**
+     * Adds a new clipping \a region to the map settings.
+     *
+     * \see clippingRegions()
+     * \see setClippingRegions()
+     *
+     * \since QGIS 3.16
+     */
+    void addClippingRegion( const QgsMapClippingRegion &region );
+
+    /**
+     * Sets the list of clipping \a regions to apply to the map.
+     *
+     * \see addClippingRegion()
+     * \see clippingRegions()
+     *
+     * \since QGIS 3.16
+     */
+    void setClippingRegions( const QList< QgsMapClippingRegion > &regions );
+
+    /**
+     * Returns the list of clipping regions to apply to the map.
+     *
+     * \see addClippingRegion()
+     * \see setClippingRegions()
+     *
+     * \since QGIS 3.16
+     */
+    QList< QgsMapClippingRegion > clippingRegions() const;
+
+    /**
+     * Sets the simplification setting to use when rendering vector layers.
+     *
+     * If the simplify \a method is enabled, it will override all other layer-specific simplification
+     * settings and will apply to all vector layers rendered for the map.
+     *
+     * This can be used to specify global simplification methods to apply during map exports,
+     * e.g. to allow vector layers to be simplified to an appropriate maximum level of detail
+     * during PDF exports.
+     *
+     * The default is to use no global simplification, and fallback to individual layer's settings instead.
+     *
+     * \see simplifyMethod()
+     *
+     * \since QGIS 3.10
+     */
+    void setSimplifyMethod( const QgsVectorSimplifyMethod &method ) { mSimplifyMethod = method; }
+
+    /**
+     * Returns the simplification settings to use when rendering vector layers.
+     *
+     * If enabled, it will override all other layer-specific simplification
+     * settings and will apply to all vector layers rendered for the map.
+     *
+     * The default is to use no global simplification, and fallback to individual layer's settings instead.
+     *
+     * \see setSimplifyMethod()
+     * \since QGIS 3.10
+     */
+    const QgsVectorSimplifyMethod &simplifyMethod() const { return mSimplifyMethod; }
+
+    /**
+     * Adds a rendered feature \a handler to use while rendering the map settings.
+     *
+     * Ownership of \a handler is NOT transferred, and it is the caller's responsibility to ensure
+     * that the handler exists for the lifetime of the map render job.
+     *
+     * \see renderedFeatureHandlers()
+     * \since QGIS 3.10
+     */
+    void addRenderedFeatureHandler( QgsRenderedFeatureHandlerInterface *handler );
+
+    /**
+     * Returns the list of rendered feature handlers to use while rendering the map settings.
+     * \see addRenderedFeatureHandler()
+     * \since QGIS 3.10
+     */
+    QList< QgsRenderedFeatureHandlerInterface * > renderedFeatureHandlers() const;
+
+    /**
+     * Returns the range of z-values which will be visible in the map.
+     *
+     * \see setZRange()
+     * \since QGIS 3.18
+     */
+    QgsDoubleRange zRange() const;
+
+    /**
+     * Sets the \a range of z-values which will be visible in the map.
+     *
+     * \see zRange()
+     * \since QGIS 3.18
+     */
+    void setZRange( const QgsDoubleRange &range );
+
   protected:
 
     double mDpi;
@@ -539,6 +779,7 @@ class CORE_EXPORT QgsMapSettings
     float mDevicePixelRatio = 1.0;
 
     QgsRectangle mExtent;
+    double mExtentBuffer = 0.0;
 
     double mRotation = 0.0;
     double mMagnificationFactor = 1.0;
@@ -547,6 +788,7 @@ class CORE_EXPORT QgsMapSettings
     QgsWeakMapLayerPointerList mLayers;
     QMap<QString, QString> mLayerStyleOverrides;
     QString mCustomRenderFlags;
+    QVariantMap mCustomRenderingFlags;
     QgsExpressionContext mExpressionContext;
 
     QgsCoordinateReferenceSystem mDestCRS;
@@ -583,6 +825,8 @@ class CORE_EXPORT QgsMapSettings
 
     QgsGeometry mLabelBoundaryGeometry;
 
+    QgsVectorSimplifyMethod mSimplifyMethod;
+
 #ifdef QGISDEBUG
     bool mHasTransformContext = false;
 #endif
@@ -592,6 +836,11 @@ class CORE_EXPORT QgsMapSettings
   private:
 
     QList< QgsLabelBlockingRegion > mLabelBlockingRegions;
+    QList< QgsMapClippingRegion > mClippingRegions;
+    QList< QgsRenderedFeatureHandlerInterface * > mRenderedFeatureHandlers;
+
+    QgsDoubleRange mZRange;
+
 };
 
 Q_DECLARE_OPERATORS_FOR_FLAGS( QgsMapSettings::Flags )
