@@ -18,14 +18,17 @@
 #ifndef QGSRANGE_H
 #define QGSRANGE_H
 
-#include "qgis.h"
 #include "qgis_sip.h"
 #include "qgis_core.h"
+#include "qgis.h"
+
+#include <QDate>
+#include <QDateTime>
 
 /**
  * \class QgsRange
  * \ingroup core
- * A template based class for storing ranges (lower to upper values).
+ * \brief A template based class for storing ranges (lower to upper values).
  *
  * QgsRange classes represent a range of values of some element type. For instance,
  * ranges of int might be used to represent integer ranges.
@@ -70,7 +73,7 @@ class QgsRange
     T upper() const { return mUpper; }
 
     /**
-     * Returns true if the lower bound is inclusive, or false if the lower
+     * Returns TRUE if the lower bound is inclusive, or FALSE if the lower
      * bound is exclusive.
      * \see lower()
      * \see includeUpper()
@@ -78,7 +81,7 @@ class QgsRange
     bool includeLower() const { return mIncludeLower; }
 
     /**
-     * Returns true if the upper bound is inclusive, or false if the upper
+     * Returns TRUE if the upper bound is inclusive, or FALSE if the upper
      * bound is exclusive.
      * \see upper()
      * \see includeLower()
@@ -86,20 +89,20 @@ class QgsRange
     bool includeUpper() const { return mIncludeUpper; }
 
     /**
-     * Returns true if the range is empty, ie the lower bound equals (or exceeds) the upper bound
+     * Returns TRUE if the range is empty, ie the lower bound equals (or exceeds) the upper bound
      * and either the bounds are exclusive.
      * \see isSingleton()
      */
     bool isEmpty() const { return mLower > mUpper || ( mUpper == mLower && !( mIncludeLower || mIncludeUpper ) ); }
 
     /**
-     * Returns true if the range consists only of a single value or instant.
+     * Returns TRUE if the range consists only of a single value or instant.
      * \see isEmpty()
      */
     bool isSingleton() const { return mLower == mUpper && ( mIncludeLower || mIncludeUpper ); }
 
     /**
-     * Returns true if this range contains another range.
+     * Returns TRUE if this range contains another range.
      * \see overlaps()
      */
     bool contains( const QgsRange<T> &other ) const
@@ -120,7 +123,7 @@ class QgsRange
     }
 
     /**
-     * Returns true if this range contains a specified \a element.
+     * Returns TRUE if this range contains a specified \a element.
      */
     bool contains( T element ) const
     {
@@ -138,7 +141,7 @@ class QgsRange
     }
 
     /**
-     * Returns true if this range overlaps another range.
+     * Returns TRUE if this range overlaps another range.
      * \see contains()
      */
     bool overlaps( const QgsRange<T> &other ) const
@@ -165,8 +168,20 @@ class QgsRange
       return false;
     }
 
+    bool operator==( const QgsRange<T> &other ) const
+    {
+      return mLower == other.mLower &&
+             mUpper == other.mUpper &&
+             mIncludeLower == other.includeLower() &&
+             mIncludeUpper == other.includeUpper();
+    }
 
-  private:
+    bool operator!=( const QgsRange<T> &other ) const
+    {
+      return ( ! operator==( other ) );
+    }
+
+  protected:
 
     T mLower;
     T mUpper;
@@ -177,31 +192,166 @@ class QgsRange
 
 
 /**
- * QgsRange which stores a range of double values.
+ * \brief QgsRange which stores a range of double values.
+ * \ingroup core
  * \see QgsIntRange
  * \see QgsDateRange
  * \see QgsDateTimeRange
  * \since QGIS 3.0
  */
-typedef QgsRange< double > QgsDoubleRange;
+class CORE_EXPORT QgsDoubleRange : public QgsRange< double >
+{
+  public:
 
+#ifndef SIP_RUN
 
+    /**
+     * Constructor for QgsDoubleRange. The \a lower and \a upper bounds are specified,
+     * and optionally whether or not these bounds are included in the range.
+     *
+     * The default values for \a lower and \a upper construct an infinite range (see isInfinite()).
+     *
+     * \since QGIS 3.18
+     */
+    QgsDoubleRange( double lower = std::numeric_limits< double >::lowest(),
+                    double upper = std::numeric_limits< double >::max(),
+                    bool includeLower = true, bool includeUpper = true )
+      : QgsRange( lower, upper, includeLower, includeUpper )
+    {}
+#else
+
+    /**
+     * Constructor for QgsDoubleRange. The \a lower and \a upper bounds are specified,
+     * and optionally whether or not these bounds are included in the range.
+     */
+    QgsDoubleRange( double lower,
+                    double upper,
+                    bool includeLower = true, bool includeUpper = true )
+      : QgsRange( lower, upper, includeLower, includeUpper )
+    {}
+
+    /**
+     * Constructor for QgsDoubleRange containing an infinite range (see isInfinite()).
+     *
+     * \since QGIS 3.18
+     */
+    QgsDoubleRange()
+      : QgsRange( std::numeric_limits< double >::lowest(), std::numeric_limits< double >::max(), true, true )
+    {}
+#endif
+
+    /**
+     * Returns TRUE if the range consists of all possible values.
+     * \since QGIS 3.18
+     */
+    bool isInfinite() const
+    {
+      return lower() == std::numeric_limits< double >::lowest() && upper() == std::numeric_limits< double >::max();
+    }
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString str = QStringLiteral( "<QgsDoubleRange: %1%2, %3%4>" ).arg( sipCpp->includeLower() ? QStringLiteral( "[" ) : QStringLiteral( "(" ) )
+                  .arg( sipCpp->lower() )
+                  .arg( sipCpp->upper() )
+                  .arg( sipCpp->includeUpper() ? QStringLiteral( "]" ) : QStringLiteral( ")" ) );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
+    bool operator==( const QgsDoubleRange &other ) const
+    {
+      return qgsDoubleNear( mLower, other.mLower ) &&
+             qgsDoubleNear( mUpper, other.mUpper ) &&
+             mIncludeLower == other.includeLower() &&
+             mIncludeUpper == other.includeUpper();
+    }
+
+    bool operator!=( const QgsDoubleRange &other ) const
+    {
+      return ( ! operator==( other ) );
+    }
+
+};
 
 
 /**
- * QgsRange which stores a range of integer values.
+ * \brief QgsRange which stores a range of integer values.
+ * \ingroup core
  * \see QgsDoubleRange
  * \see QgsDateRange
  * \see QgsDateTimeRange
  * \since QGIS 3.0
  */
-typedef QgsRange< int > QgsIntRange;
+class CORE_EXPORT QgsIntRange : public QgsRange< int >
+{
+  public:
+
+#ifndef SIP_RUN
+
+    /**
+     * Constructor for QgsIntRange. The \a lower and \a upper bounds are specified,
+     * and optionally whether or not these bounds are included in the range.
+     *
+     * The default values for \a lower and \a upper construct an infinite range (see isInfinite()).
+     *
+     * \since QGIS 3.18
+     */
+    QgsIntRange( int lower = std::numeric_limits< int >::lowest(),
+                 int upper = std::numeric_limits< int >::max(),
+                 bool includeLower = true, bool includeUpper = true )
+      : QgsRange( lower, upper, includeLower, includeUpper )
+    {}
+#else
+
+    /**
+     * Constructor for QgsIntRange. The \a lower and \a upper bounds are specified,
+     * and optionally whether or not these bounds are included in the range.
+     */
+    QgsIntRange( int lower,
+                 int upper,
+                 bool includeLower = true, bool includeUpper = true )
+      : QgsRange( lower, upper, includeLower, includeUpper )
+    {}
+
+    /**
+     * Constructor for QgsIntRange containing an infinite range (see isInfinite()).
+     *
+     * \since QGIS 3.18
+     */
+    QgsIntRange()
+      : QgsRange( std::numeric_limits< int >::lowest(), std::numeric_limits< int >::max(), true, true )
+    {}
+#endif
+
+    /**
+     * Returns TRUE if the range consists of all possible values.
+     * \since QGIS 3.18
+     */
+    bool isInfinite() const
+    {
+      return lower() == std::numeric_limits< int >::lowest() && upper() == std::numeric_limits< int >::max();
+    }
+
+#ifdef SIP_RUN
+    SIP_PYOBJECT __repr__();
+    % MethodCode
+    QString str = QStringLiteral( "<QgsIntRange: %1%2, %3%4>" ).arg( sipCpp->includeLower() ? QStringLiteral( "[" ) : QStringLiteral( "(" ) )
+                  .arg( sipCpp->lower() )
+                  .arg( sipCpp->upper() )
+                  .arg( sipCpp->includeUpper() ? QStringLiteral( "]" ) : QStringLiteral( ")" ) );
+    sipRes = PyUnicode_FromString( str.toUtf8().constData() );
+    % End
+#endif
+
+};
 
 
 /**
  * \class QgsTemporalRange
  * \ingroup core
- * A template based class for storing temporal ranges (beginning to end values).
+ * \brief A template based class for storing temporal ranges (beginning to end values).
  *
  * QgsTemporalRange classes represent a range of values of some temporal type. For instance,
  * ranges of QDateTime might be used to represent datetime ranges.
@@ -251,7 +401,7 @@ class QgsTemporalRange
     T end() const { return mUpper; }
 
     /**
-     * Returns true if the beginning is inclusive, or false if the beginning
+     * Returns TRUE if the beginning is inclusive, or FALSE if the beginning
      * is exclusive.
      * \see begin()
      * \see includeEnd()
@@ -259,21 +409,21 @@ class QgsTemporalRange
     bool includeBeginning() const { return mIncludeLower; }
 
     /**
-     * Returns true if the end is inclusive, or false if the end is exclusive.
+     * Returns TRUE if the end is inclusive, or FALSE if the end is exclusive.
      * \see end()
      * \see includeBeginning()
      */
     bool includeEnd() const { return mIncludeUpper; }
 
     /**
-     * Returns true if the range consists only of a single instant.
+     * Returns TRUE if the range consists only of a single instant.
      * \see isEmpty()
      * \see isInfinite()
      */
     bool isInstant() const { return mLower.isValid() && mUpper.isValid() && mLower == mUpper && ( mIncludeLower || mIncludeUpper ); }
 
     /**
-     * Returns true if the range consists of all possible values.
+     * Returns TRUE if the range consists of all possible values.
      * \see isEmpty()
      * \see isInstant()
      */
@@ -283,7 +433,7 @@ class QgsTemporalRange
     }
 
     /**
-     * Returns true if the range is empty, ie the beginning equals (or exceeds) the end
+     * Returns TRUE if the range is empty, ie the beginning equals (or exceeds) the end
      * and either of the bounds are exclusive.
      * A range with both invalid beginning and end is considered infinite and not empty.
      */
@@ -305,7 +455,7 @@ class QgsTemporalRange
     }
 
     /**
-     * Returns true if this range contains another range.
+     * Returns TRUE if this range contains another range.
      */
     bool contains( const QgsTemporalRange<T> &other ) const
     {
@@ -337,7 +487,7 @@ class QgsTemporalRange
     }
 
     /**
-     * Returns true if this range contains a specified \a element.
+     * Returns TRUE if this range contains a specified \a element.
      */
     bool contains( const T &element ) const
     {
@@ -364,7 +514,7 @@ class QgsTemporalRange
     }
 
     /**
-     * Returns true if this range overlaps another range.
+     * Returns TRUE if this range overlaps another range.
      */
     bool overlaps( const QgsTemporalRange<T> &other ) const
     {
@@ -393,12 +543,73 @@ class QgsTemporalRange
       return false;
     }
 
+    /**
+     * Extends the range in place by extending this range out to include an \a other range.
+     * If \a other is empty the range is not changed.
+     * If the range is empty and \a other is not, the range is changed and set to \a other.
+     * \see isEmpty()
+     * \returns TRUE if the range was extended
+     * \since QGIS 3.12
+     */
+    bool extend( const QgsTemporalRange<T> &other )
+    {
+      if ( other.isEmpty() )
+      {
+        return false;
+      }
+      else if ( isEmpty() )
+      {
+        mLower = other.begin();
+        mUpper = other.end();
+        mIncludeLower = other.includeBeginning();
+        mIncludeUpper = other.includeEnd();
+        return true;
+      }
+
+      // Both not empty, do some math
+      bool changed { false };
+
+      // Lower
+      if ( ! other.begin().isValid()
+           || ( begin().isValid() && other.begin() < mLower ) )
+      {
+        mLower = other.begin();
+        mIncludeLower = other.includeBeginning();
+        changed = true;
+      }
+      else if ( other.begin() == mLower && other.includeBeginning() && ! mIncludeLower )
+      {
+        mIncludeLower = true;
+        changed = true;
+      }
+
+      // Upper
+      if ( ! other.end().isValid()
+           || ( end().isValid() && other.end() > mUpper ) )
+      {
+        mUpper = other.end();
+        mIncludeUpper = other.includeEnd();
+        changed = true;
+      }
+      else if ( other.end() == mUpper && other.includeEnd() && ! mIncludeUpper )
+      {
+        mIncludeUpper = true;
+        changed = true;
+      }
+      return changed;
+    }
+
     bool operator==( const QgsTemporalRange<T> &other ) const
     {
       return mLower == other.mLower &&
              mUpper == other.mUpper &&
-             mIncludeLower == other.mIncludeLower &&
-             mIncludeUpper == other.mIncludeUpper;
+             mIncludeLower == other.includeBeginning() &&
+             mIncludeUpper == other.includeEnd();
+    }
+
+    bool operator!=( const QgsTemporalRange<T> &other ) const
+    {
+      return ( ! operator==( other ) );
     }
 
   private:
