@@ -20,10 +20,11 @@
 
 #include <QObject>
 #include "qgis_sip.h"
-#include "qgis.h"
 #include <QMap>
 #include <QFuture>
 #include <QReadWriteLock>
+#include <QSemaphore>
+#include <QElapsedTimer>
 
 #include "qgis_core.h"
 #include "qgsmaplayer.h"
@@ -45,7 +46,7 @@ typedef QList< QgsTask * > QgsTaskList;
  * task commences (ie via calling run() ).
  *
  * Long running tasks should periodically check the isCanceled() flag to detect if the task
- * has been canceled via some external event. If this flag is true then the task should
+ * has been canceled via some external event. If this flag is TRUE then the task should
  * clean up and terminate at the earliest possible convenience.
  *
  * \since QGIS 3.0
@@ -65,11 +66,13 @@ class CORE_EXPORT QgsTask : public QObject
       Complete, //!< Task successfully completed
       Terminated, //!< Task was terminated or errored
     };
+    Q_ENUM( TaskStatus )
 
     //! Task flags
     enum Flag
     {
       CanCancel = 1 << 1, //!< Task can be canceled
+      CancelWithoutPrompt = 1 << 2, //!< Task can be canceled without any users prompts, e.g. when closing a project or QGIS.
       AllFlags = CanCancel, //!< Task supports all flags
     };
     Q_DECLARE_FLAGS( Flags, Flag )
@@ -89,12 +92,19 @@ class CORE_EXPORT QgsTask : public QObject
     Flags flags() const { return mFlags; }
 
     /**
-     * Returns true if the task can be canceled.
+     * Sets the task's \a description. This must be called before adding the task to a QgsTaskManager,
+     * changing the description after queuing the task has no effect.
+     * \since QGIS 3.10
+     */
+    void setDescription( const QString &description );
+
+    /**
+     * Returns TRUE if the task can be canceled.
      */
     bool canCancel() const { return mFlags & CanCancel; }
 
     /**
-     * Returns true if the task is active, ie it is not complete and has
+     * Returns TRUE if the task is active, ie it is not complete and has
      * not been canceled.
      */
     bool isActive() const { return mOverallStatus == Running; }
@@ -200,9 +210,9 @@ class CORE_EXPORT QgsTask : public QObject
      * If \a timeout is ``0`` the thread will be blocked forever.
      * In case of a timeout, the task will still be running.
      * In case the task already is finished, the method will return immediately while
-     * returning ``true``.
+     * returning ``TRUE``.
      *
-     * The result will be false if the wait timed out and true in any other case.
+     * The result will be FALSE if the wait timed out and TRUE in any other case.
      */
     bool waitForFinished( int timeout = 30000 );
 
@@ -269,14 +279,14 @@ class CORE_EXPORT QgsTask : public QObject
      * for the duration of this method so tasks should avoid performing any
      * lengthy operations here.
      */
-    virtual void finished( bool result ) { Q_UNUSED( result ); }
+    virtual void finished( bool result ) { Q_UNUSED( result ) }
 
     /**
-     * Will return true if task should terminate ASAP. If the task reports the CanCancel
+     * Will return TRUE if task should terminate ASAP. If the task reports the CanCancel
      * flag, then derived classes' run() methods should periodically check this and
      * terminate in a safe manner.
      */
-    bool isCanceled() const { return mShouldTerminate; }
+    bool isCanceled() const;
 
   protected slots:
 
@@ -305,11 +315,18 @@ class CORE_EXPORT QgsTask : public QObject
      */
     QMutex mNotFinishedMutex;
 
+    /**
+     * This semaphore remains locked from task creation until the task actually start,
+     * it's used in waitForFinished to actually wait the task to be started.
+     */
+    QSemaphore mNotStartedMutex;
+
     //! Progress of this (parent) task alone
     double mProgress = 0.0;
     //! Overall progress of this task and all subtasks
     double mTotalProgress = 0.0;
     bool mShouldTerminate = false;
+    mutable QMutex mShouldTerminateMutex;
     int mStartCount = 0;
 
     struct SubTask
@@ -341,13 +358,10 @@ class CORE_EXPORT QgsTask : public QObject
     void completed();
 
     /**
-     * Called when the task has failed, as either a result of an internal failure or via cancelation.
+     * Called when the task has failed, as either a result of an internal failure or via cancellation.
      */
     void terminated();
 
-    void processSubTasksForCompletion();
-
-    void processSubTasksForTermination();
 
     void processSubTasksForHold();
 
@@ -355,6 +369,12 @@ class CORE_EXPORT QgsTask : public QObject
     friend class QgsTaskRunnableWrapper;
     friend class TestQgsTaskManager;
 
+  private slots:
+
+    void processSubTasksForCompletion();
+
+    void processSubTasksForTermination();
+
 };
 
 
@@ -431,7 +451,7 @@ class CORE_EXPORT QgsTaskManager : public QObject
     /**
      * Returns the task with matching ID.
      * \param id task ID
-     * \returns task if found, or nullptr
+     * \returns task if found, or NULLPTR
      */
     QgsTask *task( long id ) const;
 
@@ -457,7 +477,7 @@ class CORE_EXPORT QgsTaskManager : public QObject
      */
     void cancelAll();
 
-    //! Returns true if all dependencies for the specified task are satisfied
+    //! Returns TRUE if all dependencies for the specified task are satisfied
     bool dependenciesSatisfied( long taskId ) const;
 
     /**
@@ -468,7 +488,7 @@ class CORE_EXPORT QgsTaskManager : public QObject
 
     /**
      * Returns a list of layers on which as task is dependent. The task will automatically
-     * be canceled if any of these layers are above to be removed.
+     * be canceled if any of these layers are about to be removed.
      * \param taskId task ID
      * \returns list of layers
      * \see tasksDependentOnLayer()
@@ -575,6 +595,8 @@ class CORE_EXPORT QgsTaskManager : public QObject
       QgsTaskRunnableWrapper *runnable = nullptr;
     };
 
+    bool mInitialized = false;
+
     mutable QMutex *mTaskMutex;
 
     QMap< long, TaskInfo > mTasks;
@@ -615,7 +637,7 @@ class CORE_EXPORT QgsTaskManager : public QObject
 
     bool resolveDependencies( long firstTaskId, long currentTaskId, QSet< long > &results ) const;
 
-    //! Will return true if the specified task has circular dependencies
+    //! Will return TRUE if the specified task has circular dependencies
     bool hasCircularDependencies( long taskId ) const;
 
     friend class TestQgsTaskManager;
