@@ -19,7 +19,8 @@
 #define QGSUNITTYPES_H
 
 #include "qgis_core.h"
-#include "qgis.h"
+#include "qgis_sip.h"
+#include <QObject>
 
 /***************************************************************************
  * This class is considered CRITICAL and any change MUST be accompanied with
@@ -39,6 +40,29 @@ class CORE_EXPORT QgsUnitTypes
     Q_GADGET
 
   public:
+    //! Systems of unit measurement
+    enum SystemOfMeasurement
+    {
+      UnknownSystem = 0, //!< Unknown system of measurement
+      MetricSystem, //!< International System of Units (SI)
+      ImperialSystem, //!< British Imperial
+      USCSSystem //!< United States customary system
+    };
+    Q_ENUM( SystemOfMeasurement )
+
+    /**
+     * Unit types.
+     * \since QGIS 3.10
+     */
+    enum UnitType
+    {
+      TypeDistance = 0, //!< Distance unit
+      TypeArea, //!< Area unit
+      TypeVolume, //!< Volume unit
+      TypeUnknown, //!< Unknown unit type
+      TypeTemporal, //!< Temporal unit
+    };
+
     //! Units of distance
     enum DistanceUnit
     {
@@ -77,12 +101,32 @@ class CORE_EXPORT QgsUnitTypes
       AreaAcres, //!< Acres
       AreaSquareNauticalMiles, //!< Square nautical miles
       AreaSquareDegrees, //!< Square degrees, for planar geographic CRS area measurements
-      AreaSquareCentimeters, //! Square centimeters
-      AreaSquareMillimeters, //! Square millimeters
+      AreaSquareCentimeters, //!< Square centimeters
+      AreaSquareMillimeters, //!< Square millimeters
       AreaUnknownUnit, //!< Unknown areal unit
     };
     Q_ENUM( AreaUnit )
 
+    /**
+     * Units of volume.
+     * \since QGIS 3.10
+     */
+    enum VolumeUnit
+    {
+      VolumeCubicMeters = 0, //!< Cubic meters
+      VolumeCubicFeet, //!< Cubic feet
+      VolumeCubicYards, //!< Cubic yards
+      VolumeBarrel, //!< Barrels
+      VolumeCubicDecimeter, //!< Cubic decimeters
+      VolumeLiters, //!< Litres
+      VolumeGallonUS, //!< US Gallons
+      VolumeCubicInch, //!< Cubic inches
+      VolumeCubicCentimeter, //!< Cubic Centimeters
+      VolumeCubicDegrees, //!< Cubic degrees, for planar geographic CRS volume measurements
+      VolumeUnknownUnit, //!< Unknown volume unit
+    };
+    Q_ENUM( VolumeUnit )
+
     //! Units of angles
     enum AngleUnit
     {
@@ -92,10 +136,32 @@ class CORE_EXPORT QgsUnitTypes
       AngleMinutesOfArc, //!< Minutes of arc
       AngleSecondsOfArc, //!< Seconds of arc
       AngleTurn, //!< Turn/revolutions
+      AngleMilliradiansSI, //!< Angular milliradians (SI definition, 1/1000 of radian)
+      AngleMilNATO, //!< Angular mil (NATO definition, 6400 mil = 2PI radians)
       AngleUnknownUnit, //!< Unknown angle unit
     };
     Q_ENUM( AngleUnit )
 
+    /**
+     * Temporal units.
+     * \since QGIS 3.14
+     */
+    enum TemporalUnit
+    {
+      TemporalMilliseconds, //!< Milliseconds
+      TemporalSeconds, //!< Seconds
+      TemporalMinutes, //!< Minutes
+      TemporalHours, //!< Hours
+      TemporalDays, //!< Days
+      TemporalWeeks, //!< Weeks
+      TemporalMonths,  //!< Months
+      TemporalYears, //!< Years
+      TemporalDecades, //!< Decades
+      TemporalCenturies, //!< Centuries
+      TemporalUnknownUnit //!< Unknown time unit
+    };
+    Q_ENUM( TemporalUnit )
+
     //! Rendering size units
     enum RenderUnit
     {
@@ -103,8 +169,8 @@ class CORE_EXPORT QgsUnitTypes
       RenderMapUnits, //!< Map units
       RenderPixels, //!< Pixels
       RenderPercentage, //!< Percentage of another measurement (e.g., canvas size, feature size)
-      RenderPoints, //! points (e.g., for font sizes)
-      RenderInches, //! Inches
+      RenderPoints, //!< Points (e.g., for font sizes)
+      RenderInches, //!< Inches
       RenderUnknownUnit, //!< Mixed or unknown units
       RenderMetersInMapUnits, //!< Meters value as Map units
     };
@@ -172,12 +238,31 @@ class CORE_EXPORT QgsUnitTypes
     //! List of render units
     typedef QList<QgsUnitTypes::RenderUnit> RenderUnitList;
 
+    /**
+     * Encodes a unit \a type to a string.
+     * \returns encoded string
+     * \see decodeUnitType()
+     * \since QGIS 3.10
+     */
+    Q_INVOKABLE static QString encodeUnitType( QgsUnitTypes::UnitType type );
+
+    /**
+     * Decodes a unit type from a \a string.
+     * \param string string to decode
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
+     * \returns decoded unit type
+     * \see encodeUnitType()
+     * \since QGIS 3.10
+     */
+    Q_INVOKABLE static QgsUnitTypes::UnitType decodeUnitType( const QString &string, bool *ok SIP_OUT = nullptr );
+
+
     // DISTANCE UNITS
 
     /**
      * Returns the type for a distance unit.
      */
-    Q_INVOKABLE static DistanceUnitType unitType( QgsUnitTypes::DistanceUnit unit );
+    Q_INVOKABLE static QgsUnitTypes::DistanceUnitType unitType( QgsUnitTypes::DistanceUnit unit );
 
     /**
      * Encodes a distance unit to a string.
@@ -190,7 +275,7 @@ class CORE_EXPORT QgsUnitTypes
     /**
      * Decodes a distance unit from a string.
      * \param string string to decode
-     * \param ok optional boolean, will be set to true if string was converted successfully
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
      * \returns decoded units
      * \see encodeUnit()
      */
@@ -215,7 +300,8 @@ class CORE_EXPORT QgsUnitTypes
     /**
      * Converts a translated string to a distance unit.
      * \param string string representing a distance unit
-     * \param ok optional boolean, will be set to true if string was converted successfully
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
+     * \returns the distance unit
      * \see toString()
      */
     Q_INVOKABLE static QgsUnitTypes::DistanceUnit stringToDistanceUnit( const QString &string, bool *ok SIP_OUT = nullptr );
@@ -233,7 +319,7 @@ class CORE_EXPORT QgsUnitTypes
     /**
      * Returns the type for an areal unit.
     */
-    Q_INVOKABLE static DistanceUnitType unitType( QgsUnitTypes::AreaUnit unit );
+    Q_INVOKABLE static QgsUnitTypes::DistanceUnitType unitType( QgsUnitTypes::AreaUnit unit );
 
     /**
      * Encodes an areal unit to a string.
@@ -246,18 +332,18 @@ class CORE_EXPORT QgsUnitTypes
     /**
      * Decodes an areal unit from a string.
      * \param string string to decode
-     * \param ok optional boolean, will be set to true if string was converted successfully
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
      * \returns decoded units
      * \see encodeUnit()
     */
-    Q_INVOKABLE static AreaUnit decodeAreaUnit( const QString &string, bool *ok SIP_OUT = nullptr );
+    Q_INVOKABLE static QgsUnitTypes::AreaUnit decodeAreaUnit( const QString &string, bool *ok SIP_OUT = nullptr );
 
     /**
      * Returns a translated string representing an areal unit.
      * \param unit unit to convert to string
      * \see stringToAreaUnit()
      */
-    Q_INVOKABLE static QString toString( QgsUnitTypes::AreaUnit unit );
+    static QString toString( QgsUnitTypes::AreaUnit unit );
 
     /**
      * Returns a translated abbreviation representing an areal unit.
@@ -266,15 +352,16 @@ class CORE_EXPORT QgsUnitTypes
      *
      * \since QGIS 3.0
      */
-    Q_INVOKABLE static QString toAbbreviatedString( QgsUnitTypes::AreaUnit unit );
+    static QString toAbbreviatedString( QgsUnitTypes::AreaUnit unit );
 
     /**
      * Converts a translated string to an areal unit.
      * \param string string representing an areal unit
-     * \param ok optional boolean, will be set to true if string was converted successfully
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
+     * \returns the area unit
      * \see toString()
      */
-    Q_INVOKABLE static AreaUnit stringToAreaUnit( const QString &string, bool *ok SIP_OUT = nullptr );
+    Q_INVOKABLE static QgsUnitTypes::AreaUnit stringToAreaUnit( const QString &string, bool *ok SIP_OUT = nullptr );
 
     /**
      * Returns the conversion factor between the specified areal units.
@@ -291,6 +378,148 @@ class CORE_EXPORT QgsUnitTypes
      */
     Q_INVOKABLE static QgsUnitTypes::AreaUnit distanceToAreaUnit( QgsUnitTypes::DistanceUnit distanceUnit );
 
+    /**
+     * Converts an area unit to its corresponding distance unit, e.g., square meters to meters
+     * \param areaUnit area unit to convert
+     * \returns matching distance unit
+     * \since QGIS 3.10
+     */
+    Q_INVOKABLE static QgsUnitTypes::DistanceUnit areaToDistanceUnit( QgsUnitTypes::AreaUnit areaUnit );
+
+    // TEMPORAL UNITS
+
+    /**
+     * Encodes a temporal \a unit to a string.
+     * \returns encoded string
+     * \see decodeTemporalUnit()
+     * \since QGIS 3.14
+    */
+    Q_INVOKABLE static QString encodeUnit( QgsUnitTypes::TemporalUnit unit );
+
+    /**
+     * Decodes a temporal unit from a \a string.
+     * \param string string to decode
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
+     * \returns decoded units
+     * \see encodeUnit()
+     * \since QGIS 3.14
+    */
+    Q_INVOKABLE static QgsUnitTypes::TemporalUnit decodeTemporalUnit( const QString &string, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Returns a translated string representing a temporal \a unit.
+     * \see stringToTemporalUnit()
+     * \since QGIS 3.14
+     */
+    static QString toString( QgsUnitTypes::TemporalUnit unit );
+
+    /**
+     * Returns a translated abbreviation representing a temporal \a unit.
+     * \see stringToTemporalUnit()
+     *
+     * \since QGIS 3.14
+     */
+    static QString toAbbreviatedString( QgsUnitTypes::TemporalUnit unit );
+
+    /**
+     * Converts a translated \a string to a temporal unit.
+     * \param string string representing a volume unit
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
+     * \returns the temporal unit
+     * \see toString()
+     * \since QGIS 3.14
+     */
+    Q_INVOKABLE static QgsUnitTypes::TemporalUnit stringToTemporalUnit( const QString &string, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Returns the conversion factor between the specified temporal units.
+     *
+     * \note Conversion to or from month units assumes a 30 day month length.
+     * \note Conversion to or from year based units assumes a 365.25 day year length.
+     *
+     * \param fromUnit temporal unit to convert from
+     * \param toUnit temporal unit to convert to
+     * \returns multiplication factor to convert between units
+     * \since QGIS 3.14
+     */
+    Q_INVOKABLE static double fromUnitToUnitFactor( QgsUnitTypes::TemporalUnit fromUnit, QgsUnitTypes::TemporalUnit toUnit );
+
+    // VOLUME UNITS
+
+    /**
+     * Returns the type for an volume unit.
+     * \since QGIS 3.10
+    */
+    Q_INVOKABLE static QgsUnitTypes::DistanceUnitType unitType( QgsUnitTypes::VolumeUnit unit );
+
+    /**
+     * Encodes a volume \a unit to a string.
+     * \returns encoded string
+     * \see decodeVolumeUnit()
+     * \since QGIS 3.10
+    */
+    Q_INVOKABLE static QString encodeUnit( QgsUnitTypes::VolumeUnit unit );
+
+    /**
+     * Decodes a volume unit from a \a string.
+     * \param string string to decode
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
+     * \returns decoded units
+     * \see encodeUnit()
+     * \since QGIS 3.10
+    */
+    Q_INVOKABLE static QgsUnitTypes::VolumeUnit decodeVolumeUnit( const QString &string, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Returns a translated string representing a volume \a unit.
+     * \see stringToVolumeUnit()
+     * \since QGIS 3.10
+     */
+    static QString toString( QgsUnitTypes::VolumeUnit unit );
+
+    /**
+     * Returns a translated abbreviation representing a volume \a unit.
+     * \see stringToVolumeUnit()
+     *
+     * \since QGIS 3.10
+     */
+    static QString toAbbreviatedString( QgsUnitTypes::VolumeUnit unit );
+
+    /**
+     * Converts a translated \a string to a volume unit.
+     * \param string string representing a volume unit
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
+     * \returns the volume unit
+     * \see toString()
+     * \since QGIS 3.10
+     */
+    Q_INVOKABLE static QgsUnitTypes::VolumeUnit stringToVolumeUnit( const QString &string, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Returns the conversion factor between the specified volume units.
+     * \param fromUnit volume unit to convert from
+     * \param toUnit volume unit to convert to
+     * \returns multiplication factor to convert between units
+     * \since QGIS 3.10
+     */
+    Q_INVOKABLE static double fromUnitToUnitFactor( QgsUnitTypes::VolumeUnit fromUnit, QgsUnitTypes::VolumeUnit toUnit );
+
+    /**
+     * Converts a distance unit to its corresponding volume unit, e.g., meters to cubic meters
+     * \param distanceUnit distance unit to convert
+     * \returns matching volume unit
+     * \since QGIS 3.10
+     */
+    Q_INVOKABLE static QgsUnitTypes::VolumeUnit distanceToVolumeUnit( QgsUnitTypes::DistanceUnit distanceUnit );
+
+    /**
+     * Converts a volume unit to its corresponding distance unit, e.g., cubic meters to meters
+     * \param volumeUnit volume unit to convert
+     * \returns matching distance unit
+     * \since QGIS 3.10
+     */
+    Q_INVOKABLE static QgsUnitTypes::DistanceUnit volumeToDistanceUnit( QgsUnitTypes::VolumeUnit volumeUnit );
+
     // ANGULAR UNITS
 
     /**
@@ -304,7 +533,7 @@ class CORE_EXPORT QgsUnitTypes
     /**
      * Decodes an angular unit from a string.
      * \param string string to decode
-     * \param ok optional boolean, will be set to true if string was converted successfully
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
      * \returns decoded units
      * \see encodeUnit()
     */
@@ -314,7 +543,7 @@ class CORE_EXPORT QgsUnitTypes
      * Returns a translated string representing an angular unit.
      * \param unit unit to convert to string
      */
-    Q_INVOKABLE static QString toString( QgsUnitTypes::AngleUnit unit );
+    static QString toString( QgsUnitTypes::AngleUnit unit );
 
     /**
      * Returns the conversion factor between the specified angular units.
@@ -361,7 +590,7 @@ class CORE_EXPORT QgsUnitTypes
      * \param distance distance to format
      * \param decimals number of decimal places to show
      * \param unit unit of distance
-     * \param keepBaseUnit set to false to allow conversion of large distances to more suitable units, e.g., meters to
+     * \param keepBaseUnit set to FALSE to allow conversion of large distances to more suitable units, e.g., meters to
      * kilometers
      * \returns formatted distance string
      * \see formatArea()
@@ -374,7 +603,7 @@ class CORE_EXPORT QgsUnitTypes
      * \param area area to format
      * \param decimals number of decimal places to show
      * \param unit unit of area
-     * \param keepBaseUnit set to false to allow conversion of large areas to more suitable units, e.g., square meters to
+     * \param keepBaseUnit set to FALSE to allow conversion of large areas to more suitable units, e.g., square meters to
      * square kilometers
      * \returns formatted area string
      * \see formatDistance()
@@ -395,17 +624,26 @@ class CORE_EXPORT QgsUnitTypes
     /**
      * Decodes a render unit from a string.
      * \param string string to decode
-     * \param ok optional boolean, will be set to true if string was converted successfully
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
      * \returns decoded units
      * \see encodeUnit()
      */
     Q_INVOKABLE static QgsUnitTypes::RenderUnit decodeRenderUnit( const QString &string, bool *ok SIP_OUT = nullptr );
 
+
     /**
      * Returns a translated string representing a render \a unit.
      * \since QGIS 3.0
      */
-    Q_INVOKABLE static QString toString( QgsUnitTypes::RenderUnit unit );
+    static QString toString( QgsUnitTypes::RenderUnit unit );
+
+    /**
+     * Returns a translated abbreviation representing a render unit.
+     * \param unit unit to convert to string
+     *
+     * \since QGIS 3.8
+     */
+    static QString toAbbreviatedString( QgsUnitTypes::RenderUnit unit );
 
 
     // LAYOUT UNITS
@@ -422,7 +660,7 @@ class CORE_EXPORT QgsUnitTypes
     /**
      * Decodes a layout unit from a string.
      * \param string string to decode
-     * \param ok optional boolean, will be set to true if string was converted successfully
+     * \param ok optional boolean, will be set to TRUE if string was converted successfully
      * \returns decoded units
      * \see encodeUnit()
      * \since QGIS 3.0
@@ -441,14 +679,14 @@ class CORE_EXPORT QgsUnitTypes
      *
      * \since QGIS 3.0
      */
-    Q_INVOKABLE static QString toAbbreviatedString( QgsUnitTypes::LayoutUnit unit );
+    static QString toAbbreviatedString( QgsUnitTypes::LayoutUnit unit );
 
     /**
      * Returns a translated string representing a layout \a unit.
      *
      * \since QGIS 3.0
      */
-    Q_INVOKABLE static QString toString( QgsUnitTypes::LayoutUnit unit );
+    static QString toString( QgsUnitTypes::LayoutUnit unit );
 
 };
 
