@@ -20,8 +20,6 @@
 __author__ = 'Sandro Santilli'
 __date__ = 'May 2017'
 __copyright__ = '(C) 2017, Sandro Santilli'
-# This will get replaced with a git SHA1 when you do a git archive
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
 
 import os
 import re
@@ -45,7 +43,13 @@ class TestDBManagerPostgisPlugin(unittest.TestCase):
     @classmethod
     def setUpClass(self):
         self.old_pgdatabase_env = os.environ.get('PGDATABASE')
-        self.testdb = os.environ.get('QGIS_PGTEST_DB') or 'qgis_test'
+        # QGIS_PGTEST_DB contains the full connection string and not only the DB name!
+        QGIS_PGTEST_DB = os.environ.get('QGIS_PGTEST_DB')
+        if QGIS_PGTEST_DB is not None:
+            test_uri = QgsDataSourceUri(QGIS_PGTEST_DB)
+            self.testdb = test_uri.database()
+        else:
+            self.testdb = 'qgis_test'
         os.environ['PGDATABASE'] = self.testdb
 
         # Create temporary service file
@@ -68,34 +72,36 @@ class TestDBManagerPostgisPlugin(unittest.TestCase):
         # Remove temporary service file
         os.unlink(self.tmpservicefile)
 
-    # See https://issues.qgis.org/issues/16625
+    # See https://github.com/qgis/QGIS/issues/24525
 
-    def test_rasterTableGdalURI(self):
+    def test_rasterTableURI(self):
 
-        def check_rasterTableGdalURI(expected_dbname):
+        def check_rasterTableURI(expected_dbname):
             tables = database.tables()
             raster_tables_count = 0
             for tab in tables:
                 if tab.type == Table.RasterType:
                     raster_tables_count += 1
-                    gdalUri = tab.gdalUri()
-                    m = re.search(' dbname=\'([^ ]*)\' ', gdalUri)
+                    uri = tab.uri()
+                    m = re.search(' dbname=\'([^ ]*)\' ', uri)
                     self.assertTrue(m)
                     actual_dbname = m.group(1)
                     self.assertEqual(actual_dbname, expected_dbname)
-                #print(tab.type)
-                #print(tab.quotedName())
-                #print(tab)
+                # print(tab.type)
+                # print(tab.quotedName())
+                # print(tab)
 
             # We need to make sure a database is created with at
             # least one raster table !
-            self.assertEqual(raster_tables_count, 1)
+            self.assertGreaterEqual(raster_tables_count, 1)
 
-        obj = QObject() # needs to be kept alive
+        obj = QObject()  # needs to be kept alive
+        obj.connectionName = lambda: 'fake'
+        obj.providerName = lambda: 'postgres'
 
         # Test for empty URI
-        # See https://issues.qgis.org/issues/16625
-        # and https://issues.qgis.org/issues/10600
+        # See https://github.com/qgis/QGIS/issues/24525
+        # and https://github.com/qgis/QGIS/issues/19005
 
         expected_dbname = self.testdb
         os.environ['PGDATABASE'] = expected_dbname
@@ -109,10 +115,10 @@ class TestDBManagerPostgisPlugin(unittest.TestCase):
         self.assertEqual(uri.database(), expected_dbname)
         self.assertEqual(uri.service(), '')
 
-        check_rasterTableGdalURI(expected_dbname)
+        check_rasterTableURI(expected_dbname)
 
         # Test for service-only URI
-        # See https://issues.qgis.org/issues/16626
+        # See https://github.com/qgis/QGIS/issues/24526
 
         os.environ['PGDATABASE'] = 'fake'
         database = PGDatabase(obj, QgsDataSourceUri('service=dbmanager'))
@@ -124,12 +130,14 @@ class TestDBManagerPostgisPlugin(unittest.TestCase):
         self.assertEqual(uri.database(), '')
         self.assertEqual(uri.service(), 'dbmanager')
 
-        check_rasterTableGdalURI(expected_dbname)
+        check_rasterTableURI(expected_dbname)
 
-    # See http://issues.qgis.org/issues/16833
+    # See https://github.com/qgis/QGIS/issues/24732
     def test_unicodeInQuery(self):
         os.environ['PGDATABASE'] = self.testdb
-        obj = QObject() # needs to be kept alive
+        obj = QObject()  # needs to be kept alive
+        obj.connectionName = lambda: 'fake'
+        obj.providerName = lambda: 'postgres'
         database = PGDatabase(obj, QgsDataSourceUri())
         self.assertIsInstance(database, PGDatabase)
         # SQL as string literal
