@@ -21,10 +21,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'November 2012'
 __copyright__ = '(C) 2012, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
-
 from qgis.core import (QgsProcessing,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterString,
@@ -32,12 +28,18 @@ from qgis.core import (QgsProcessing,
                        QgsProcessingParameterCrs,
                        QgsProcessingParameterField,
                        QgsProcessingParameterExtent,
-                       QgsProcessingParameterBoolean)
+                       QgsProcessingParameterBoolean,
+                       QgsProcessingParameterProviderConnection,
+                       QgsProcessingParameterDatabaseSchema,
+                       QgsProcessingParameterDatabaseTable,
+                       QgsProviderRegistry,
+                       QgsProcessingException,
+                       QgsProviderConnectionException,
+                       QgsDataSourceUri)
 
 from processing.algs.gdal.GdalAlgorithm import GdalAlgorithm
 from processing.algs.gdal.GdalUtils import GdalUtils
 
-from processing.tools.postgis import uri_from_name, GeoDB
 from processing.tools.system import isWindows
 
 
@@ -47,7 +49,7 @@ class Ogr2OgrToPostGisList(GdalAlgorithm):
     SHAPE_ENCODING = 'SHAPE_ENCODING'
     GTYPE = 'GTYPE'
     GEOMTYPE = ['', 'NONE', 'GEOMETRY', 'POINT', 'LINESTRING', 'POLYGON', 'GEOMETRYCOLLECTION', 'MULTIPOINT',
-                'MULTIPOLYGON', 'MULTILINESTRING']
+                'MULTIPOLYGON', 'MULTILINESTRING', 'CIRCULARSTRING', 'COMPOUNDCURVE', 'CURVEPOLYGON', 'MULTICURVE', 'MULTISURFACE']
     S_SRS = 'S_SRS'
     T_SRS = 'T_SRS'
     A_SRS = 'A_SRS'
@@ -62,7 +64,7 @@ class Ogr2OgrToPostGisList(GdalAlgorithm):
     PRIMARY_KEY = 'PRIMARY_KEY'
     GEOCOLUMN = 'GEOCOLUMN'
     DIM = 'DIM'
-    DIMLIST = ['2', '3']
+    DIMLIST = ['2', '3', '4']
     SIMPLIFY = 'SIMPLIFY'
     SEGMENTIZE = 'SEGMENTIZE'
     SPAT = 'SPAT'
@@ -84,12 +86,9 @@ class Ogr2OgrToPostGisList(GdalAlgorithm):
 
     def initAlgorithm(self, config=None):
 
-        db_param = QgsProcessingParameterString(
+        db_param = QgsProcessingParameterProviderConnection(
             self.DATABASE,
-            self.tr('Database (connection name)'))
-        db_param.setMetadata({
-            'widget_wrapper': {
-                'class': 'processing.gui.wrappers_postgis.ConnectionWidgetWrapper'}})
+            self.tr('Database (connection name)'), 'postgres')
         self.addParameter(db_param)
         self.addParameter(QgsProcessingParameterFeatureSource(self.INPUT,
                                                               self.tr('Input layer'),
@@ -100,29 +99,22 @@ class Ogr2OgrToPostGisList(GdalAlgorithm):
                                                      self.tr('Output geometry type'), options=self.GEOMTYPE,
                                                      defaultValue=0))
         self.addParameter(QgsProcessingParameterCrs(self.A_SRS,
-                                                    self.tr('Assign an output CRS'), defaultValue='', optional=False))
+                                                    self.tr('Assign an output CRS'), defaultValue='', optional=True))
         self.addParameter(QgsProcessingParameterCrs(self.T_SRS,
                                                     self.tr('Reproject to this CRS on output '), defaultValue='',
                                                     optional=True))
         self.addParameter(QgsProcessingParameterCrs(self.S_SRS,
                                                     self.tr('Override source CRS'), defaultValue='', optional=True))
 
-        schema_param = QgsProcessingParameterString(
+        schema_param = QgsProcessingParameterDatabaseSchema(
             self.SCHEMA,
-            self.tr('Schema (schema name)'), 'public', False, True)
-        schema_param.setMetadata({
-            'widget_wrapper': {
-                'class': 'processing.gui.wrappers_postgis.SchemaWidgetWrapper',
-                'connection_param': self.DATABASE}})
+            self.tr('Schema (schema name)'), defaultValue='public', connectionParameterName=self.DATABASE, optional=True)
         self.addParameter(schema_param)
 
-        table_param = QgsProcessingParameterString(
+        table_param = QgsProcessingParameterDatabaseTable(
             self.TABLE,
-            self.tr('Table to import to (leave blank to use layer name)'), '', False, True)
-        table_param.setMetadata({
-            'widget_wrapper': {
-                'class': 'processing.gui.wrappers_postgis.TableWidgetWrapper',
-                'schema_param': self.SCHEMA}})
+            self.tr('Table to import to (leave blank to use layer name)'), defaultValue=None, connectionParameterName=self.DATABASE,
+            schemaParameterName=self.SCHEMA, optional=True, allowNewTableNames=True)
         self.addParameter(table_param)
 
         self.addParameter(QgsProcessingParameterString(self.PK,
@@ -206,19 +198,27 @@ class Ogr2OgrToPostGisList(GdalAlgorithm):
         return 'vectormiscellaneous'
 
     def getConsoleCommands(self, parameters, context, feedback, executing=True):
-        connection = self.parameterAsString(parameters, self.DATABASE, context)
-        uri = uri_from_name(connection)
-        if executing:
-            # to get credentials input when needed
-            uri = GeoDB(uri=uri).uri
+        connection_name = self.parameterAsConnectionName(parameters, self.DATABASE, context)
+        if not connection_name:
+            raise QgsProcessingException(
+                self.tr('No connection specified'))
+
+        # resolve connection details to uri
+        try:
+            md = QgsProviderRegistry.instance().providerMetadata('postgres')
+            conn = md.createConnection(connection_name)
+        except QgsProviderConnectionException:
+            raise QgsProcessingException(self.tr('Could not retrieve connection details for {}').format(connection_name))
+
+        uri = conn.uri()
 
         ogrLayer, layername = self.getOgrCompatibleSource(self.INPUT, parameters, context, feedback, executing)
         shapeEncoding = self.parameterAsString(parameters, self.SHAPE_ENCODING, context)
         ssrs = self.parameterAsCrs(parameters, self.S_SRS, context)
         tsrs = self.parameterAsCrs(parameters, self.T_SRS, context)
         asrs = self.parameterAsCrs(parameters, self.A_SRS, context)
-        table = self.parameterAsString(parameters, self.TABLE, context)
-        schema = self.parameterAsString(parameters, self.SCHEMA, context)
+        table = self.parameterAsDatabaseTableName(parameters, self.TABLE, context)
+        schema = self.parameterAsSchema(parameters, self.SCHEMA, context)
         pk = self.parameterAsString(parameters, self.PK, context)
         pkstring = "-lco FID=" + pk
         primary_key = self.parameterAsString(parameters, self.PRIMARY_KEY, context)
@@ -229,20 +229,20 @@ class Ogr2OgrToPostGisList(GdalAlgorithm):
         simplify = self.parameterAsString(parameters, self.SIMPLIFY, context)
         segmentize = self.parameterAsString(parameters, self.SEGMENTIZE, context)
         spat = self.parameterAsExtent(parameters, self.SPAT, context)
-        clip = self.parameterAsBool(parameters, self.CLIP, context)
+        clip = self.parameterAsBoolean(parameters, self.CLIP, context)
         where = self.parameterAsString(parameters, self.WHERE, context)
         wherestring = '-where "' + where + '"'
         gt = self.parameterAsString(parameters, self.GT, context)
-        overwrite = self.parameterAsBool(parameters, self.OVERWRITE, context)
-        append = self.parameterAsBool(parameters, self.APPEND, context)
-        addfields = self.parameterAsBool(parameters, self.ADDFIELDS, context)
-        launder = self.parameterAsBool(parameters, self.LAUNDER, context)
+        overwrite = self.parameterAsBoolean(parameters, self.OVERWRITE, context)
+        append = self.parameterAsBoolean(parameters, self.APPEND, context)
+        addfields = self.parameterAsBoolean(parameters, self.ADDFIELDS, context)
+        launder = self.parameterAsBoolean(parameters, self.LAUNDER, context)
         launderstring = "-lco LAUNDER=NO"
-        index = self.parameterAsBool(parameters, self.INDEX, context)
+        index = self.parameterAsBoolean(parameters, self.INDEX, context)
         indexstring = "-lco SPATIAL_INDEX=OFF"
-        skipfailures = self.parameterAsBool(parameters, self.SKIPFAILURES, context)
-        promotetomulti = self.parameterAsBool(parameters, self.PROMOTETOMULTI, context)
-        precision = self.parameterAsBool(parameters, self.PRECISION, context)
+        skipfailures = self.parameterAsBoolean(parameters, self.SKIPFAILURES, context)
+        promotetomulti = self.parameterAsBoolean(parameters, self.PROMOTETOMULTI, context)
+        precision = self.parameterAsBoolean(parameters, self.PRECISION, context)
         options = self.parameterAsString(parameters, self.OPTIONS, context)
 
         arguments = []
@@ -251,14 +251,14 @@ class Ogr2OgrToPostGisList(GdalAlgorithm):
         if shapeEncoding:
             arguments.append('--config')
             arguments.append('SHAPE_ENCODING')
-            arguments.append('"' + shapeEncoding + '"')
+            arguments.append(shapeEncoding)
         arguments.append('-f')
         arguments.append('PostgreSQL')
-        arguments.append('PG:"')
-        for token in uri.connectionInfo(executing).split(' '):
-            arguments.append(token)
-        arguments.append('active_schema={}'.format(schema or 'public'))
-        arguments.append('"')
+
+        connection_parts = QgsDataSourceUri(uri).connectionInfo(executing).split(' ')
+        connection_parts.append('active_schema={}'.format(schema or 'public'))
+        arguments.append('PG:{}'.format(' '.join(connection_parts)))
+
         arguments.append(dimstring)
         arguments.append(ogrLayer)
         arguments.append(layername)
