@@ -21,14 +21,11 @@ __author__ = 'Victor Olaya'
 __date__ = 'February 2015'
 __copyright__ = '(C) 2012-2015, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '2f64a3c4e74022c5555ee19862f034ffacc2b5fe'
-
 import sys
 import os
 import re
 import uuid
+import math
 import importlib
 
 from qgis.PyQt.QtCore import QCoreApplication, QUrl
@@ -36,7 +33,8 @@ from qgis.PyQt.QtCore import QCoreApplication, QUrl
 from qgis.core import (Qgis,
                        QgsRasterLayer,
                        QgsApplication,
-                       QgsMapLayer,
+                       QgsMapLayerType,
+                       QgsCoordinateReferenceSystem,
                        QgsProcessingUtils,
                        QgsProcessing,
                        QgsMessageLog,
@@ -44,6 +42,7 @@ from qgis.core import (Qgis,
                        QgsProcessingAlgorithm,
                        QgsProcessingParameterDefinition,
                        QgsProcessingException,
+                       QgsProcessingParameterCrs,
                        QgsProcessingParameterExtent,
                        QgsProcessingParameterEnum,
                        QgsProcessingParameterNumber,
@@ -51,6 +50,7 @@ from qgis.core import (Qgis,
                        QgsProcessingParameterField,
                        QgsProcessingParameterPoint,
                        QgsProcessingParameterBoolean,
+                       QgsProcessingParameterRange,
                        QgsProcessingParameterFeatureSource,
                        QgsProcessingParameterVectorLayer,
                        QgsProcessingParameterRasterLayer,
@@ -64,11 +64,16 @@ from qgis.core import (Qgis,
                        QgsVectorLayer,
                        QgsProviderRegistry)
 from qgis.utils import iface
-from osgeo import ogr
+
+import warnings
+
+with warnings.catch_warnings():
+    warnings.filterwarnings("ignore", category=DeprecationWarning)
+    from osgeo import ogr
 
 from processing.core.ProcessingConfig import ProcessingConfig
 
-from processing.core.parameters import (getParameterFromString)
+from processing.core.parameters import getParameterFromString
 
 from .Grass7Utils import Grass7Utils
 
@@ -79,7 +84,6 @@ pluginPath = os.path.normpath(os.path.join(
 
 
 class Grass7Algorithm(QgsProcessingAlgorithm):
-
     GRASS_OUTPUT_TYPE_PARAMETER = 'GRASS_OUTPUT_TYPE_PARAMETER'
     GRASS_MIN_AREA_PARAMETER = 'GRASS_MIN_AREA_PARAMETER'
     GRASS_SNAP_TOLERANCE_PARAMETER = 'GRASS_SNAP_TOLERANCE_PARAMETER'
@@ -90,6 +94,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
     GRASS_RASTER_FORMAT_META = 'GRASS_RASTER_FORMAT_META'
     GRASS_VECTOR_DSCO = 'GRASS_VECTOR_DSCO'
     GRASS_VECTOR_LCO = 'GRASS_VECTOR_LCO'
+    GRASS_VECTOR_EXPORT_NOCAT = 'GRASS_VECTOR_EXPORT_NOCAT'
 
     OUTPUT_TYPES = ['auto', 'point', 'line', 'area']
     QGIS_OUTPUT_TYPES = {QgsProcessing.TypeVectorAnyGeometry: 'auto',
@@ -112,6 +117,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         self.commands = []
         self.outputCommands = []
         self.exportedLayers = {}
+        self.fileOutputs = {}
         self.descriptionFile = descriptionfile
 
         # Default GRASS parameters
@@ -122,6 +128,9 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         self.minArea = None
         self.alignToResolution = None
 
+        # destination Crs for combineLayerExtents, will be set from layer or mapSettings
+        self.destination_crs = QgsCoordinateReferenceSystem()
+
         # Load parameters from a description file
         self.defineCharacteristicsFromFile()
         self.numExportedLayers = 0
@@ -202,6 +211,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                 self._name = self.grass7Name
             else:
                 self._name = line[:line.find(' ')].lower()
+            self._short_description = QCoreApplication.translate("GrassAlgorithm", line)
             self._display_name = self._name
             # Read the grass group
             line = lines.readline().strip('\n').strip()
@@ -218,7 +228,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                     line = line.strip('\n').strip()
                     if line.startswith('Hardcoded'):
                         self.hardcodedStrings.append(line[len('Hardcoded|'):])
-                    parameter = getParameterFromString(line)
+                    parameter = getParameterFromString(line, "GrassAlgorithm")
                     if parameter is not None:
                         self.params.append(parameter)
                         if isinstance(parameter, (QgsProcessingParameterVectorLayer, QgsProcessingParameterFeatureSource)):
@@ -297,7 +307,8 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
             # Add an optional output type
             param = QgsProcessingParameterEnum(self.GRASS_OUTPUT_TYPE_PARAMETER,
                                                self.tr('v.out.ogr output type'),
-                                               self.OUTPUT_TYPES)
+                                               self.OUTPUT_TYPES,
+                                               defaultValue=0)
             param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
             self.params.append(param)
 
@@ -319,6 +330,15 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
             param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
             self.params.append(param)
 
+            # Add a -c flag for export
+            param = QgsProcessingParameterBoolean(
+                self.GRASS_VECTOR_EXPORT_NOCAT,
+                self.tr('Also export features without category (not labeled). Otherwise only features with category are exported'),
+                False
+            )
+            param.setFlags(param.flags() | QgsProcessingParameterDefinition.FlagAdvanced)
+            self.params.append(param)
+
     def getDefaultCellSize(self):
         """
         Determine a default cell size from all the raster layers.
@@ -361,9 +381,9 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                                                  self.GRASS_OUTPUT_TYPE_PARAMETER,
                                                  context)
         # GRASS align to resolution
-        self.alignToResolution = self.parameterAsBool(parameters,
-                                                      self.GRASS_REGION_ALIGN_TO_RESOLUTION,
-                                                      context)
+        self.alignToResolution = self.parameterAsBoolean(parameters,
+                                                         self.GRASS_REGION_ALIGN_TO_RESOLUTION,
+                                                         context)
 
     def processAlgorithm(self, original_parameters, context, feedback):
         if isWindows():
@@ -380,6 +400,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         self.commands = []
         self.outputCommands = []
         self.exportedLayers = {}
+        self.fileOutputs = {}
 
         # If GRASS session has been created outside of this algorithm then
         # get the list of layers loaded in GRASS otherwise start a new
@@ -424,9 +445,15 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         for out in self.outputDefinitions():
             outName = out.name()
             if outName in parameters:
-                outputs[outName] = parameters[outName]
+                if outName in self.fileOutputs:
+                    print('ADD', outName)
+                    print('VAL', parameters[outName])
+                    print('VAL 2', self.fileOutputs[outName])
+                    outputs[outName] = self.fileOutputs[outName]
+                else:
+                    outputs[outName] = parameters[outName]
                 if isinstance(out, QgsProcessingOutputHtml):
-                    self.convertToHtml(parameters[outName])
+                    self.convertToHtml(self.fileOutputs[outName])
 
         return outputs
 
@@ -468,15 +495,15 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                 for idx, layer in enumerate(layers):
                     layerName = '{}_{}'.format(paramName, idx)
                     # Add a raster layer
-                    if layer.type() == QgsMapLayer.RasterLayer:
+                    if layer.type() == QgsMapLayerType.RasterLayer:
                         self.loadRasterLayer(layerName, layer)
                     # Add a vector layer
-                    elif layer.type() == QgsMapLayer.VectorLayer:
+                    elif layer.type() == QgsMapLayerType.VectorLayer:
                         self.loadVectorLayer(layerName, layer, external=None, feedback=feedback)
 
-        self.postInputs()
+        self.postInputs(context)
 
-    def postInputs(self):
+    def postInputs(self, context):
         """
         After layer imports, we need to update some internal parameters
         """
@@ -485,7 +512,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
 
         # Build GRASS region
         if self.region.isEmpty():
-            self.region = QgsProcessingUtils.combineLayerExtents(self.inputLayers)
+            self.region = QgsProcessingUtils.combineLayerExtents(self.inputLayers, self.destination_crs, context)
         command = 'g.region n={} s={} e={} w={}'.format(
             self.region.yMaximum(), self.region.yMinimum(),
             self.region.xMaximum(), self.region.xMinimum()
@@ -533,7 +560,8 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                              self.GRASS_RASTER_FORMAT_OPT,
                              self.GRASS_RASTER_FORMAT_META,
                              self.GRASS_VECTOR_DSCO,
-                             self.GRASS_VECTOR_LCO]:
+                             self.GRASS_VECTOR_LCO,
+                             self.GRASS_VECTOR_EXPORT_NOCAT]:
                 continue
 
             # Raster and vector layers
@@ -557,7 +585,7 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                 value = ','.join(values)
             # For booleans, we just add the parameter name
             elif isinstance(param, QgsProcessingParameterBoolean):
-                if self.parameterAsBool(parameters, paramName, context):
+                if self.parameterAsBoolean(parameters, paramName, context):
                     command += ' {}'.format(paramName)
             # For Extents, remove if the value is null
             elif isinstance(param, QgsProcessingParameterExtent):
@@ -600,6 +628,17 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
             elif isinstance(param, (QgsProcessingParameterNumber,
                                     QgsProcessingParameterPoint)):
                 value = self.parameterAsString(parameters, paramName, context)
+            elif isinstance(param, QgsProcessingParameterRange):
+                v = self.parameterAsRange(parameters, paramName, context)
+                if (param.flags() & QgsProcessingParameterDefinition.FlagOptional) and (math.isnan(v[0]) or math.isnan(v[1])):
+                    continue
+                else:
+                    value = '{},{}'.format(v[0], v[1])
+            elif isinstance(param, QgsProcessingParameterCrs):
+                if self.parameterAsCrs(parameters, paramName, context):
+                    # TODO: ideally we should be exporting to WKT here, but it seems not all grass algorithms
+                    # will accept a wkt string for a crs value (e.g. r.tileset)
+                    value = '"{}"'.format(self.parameterAsCrs(parameters, paramName, context).toProj())
             # For everything else, we assume that it is a string
             else:
                 value = '"{}"'.format(
@@ -618,17 +657,18 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
                 # For File destination
                 if isinstance(out, QgsProcessingParameterFileDestination):
                     if outName in parameters and parameters[outName] is not None:
+                        outPath = self.parameterAsFileOutput(parameters, outName, context)
+                        self.fileOutputs[outName] = outPath
                         # for HTML reports, we need to redirect stdout
                         if out.defaultFileExtension().lower() == 'html':
-                            command += ' > "{}"'.format(
-                                self.parameterAsFileOutput(
-                                    parameters, outName, context)
-                            )
+                            if outName == 'html':
+                                # for "fake" outputs redirect command stdout
+                                command += ' > "{}"'.format(outPath)
+                            else:
+                                # for real outputs only output itself should be redirected
+                                command += ' {}=- > "{}"'.format(outName, outPath)
                         else:
-                            command += ' {}="{}"'.format(
-                                outName,
-                                self.parameterAsFileOutput(
-                                    parameters, outName, context))
+                            command += ' {}="{}"'.format(outName, outPath)
                 # For folders destination
                 elif isinstance(out, QgsProcessingParameterFolderDestination):
                     # We need to add a unique temporary basename
@@ -677,29 +717,31 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
             elif isinstance(out, QgsProcessingParameterFolderDestination):
                 self.exportRasterLayersIntoDirectory(outName, parameters, context)
 
-    def loadRasterLayerFromParameter(self, name, parameters, context, external=True, band=1):
+    def loadRasterLayerFromParameter(self, name, parameters, context, external=None, band=1):
         """
         Creates a dedicated command to load a raster into
         the temporary GRASS DB.
         :param name: name of the parameter.
         :param parameters: algorithm parameters dict.
         :param context: algorithm context.
-        :param external: True if using r.external.
+        :param external: use r.external if True, r.in.gdal otherwise.
         :param band: imports only specified band. None for all bands.
         """
         layer = self.parameterAsRasterLayer(parameters, name, context)
         self.loadRasterLayer(name, layer, external, band)
 
-    def loadRasterLayer(self, name, layer, external=True, band=1, destName=None):
+    def loadRasterLayer(self, name, layer, external=None, band=1, destName=None):
         """
         Creates a dedicated command to load a raster into
         the temporary GRASS DB.
         :param name: name of the parameter.
         :param layer: QgsMapLayer for the raster layer.
-        :param external: True if using r.external.
+        :param external: use r.external if True, r.in.gdal if False.
         :param band: imports only specified band. None for all bands.
         :param destName: force the destination name of the raster.
         """
+        if external is None:
+            external = ProcessingConfig.getSetting(Grass7Utils.GRASS_USE_REXTERNAL)
         self.inputLayers.append(layer)
         self.setSessionProjectionFromLayer(layer)
         if not destName:
@@ -732,6 +774,8 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         metaOpt = self.parameterAsString(parameters, self.GRASS_RASTER_FORMAT_META, context)
         self.exportRasterLayer(grassName, fileName, colorTable, outFormat, createOpt, metaOpt)
 
+        self.fileOutputs[name] = fileName
+
     def exportRasterLayer(self, grassName, fileName,
                           colorTable=True, outFormat='GTiff',
                           createOpt=None,
@@ -904,10 +948,13 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         outFormat = QgsVectorFileWriter.driverForExtension(os.path.splitext(fileName)[1]).replace(' ', '_')
         dsco = self.parameterAsString(parameters, self.GRASS_VECTOR_DSCO, context)
         lco = self.parameterAsString(parameters, self.GRASS_VECTOR_LCO, context)
-        self.exportVectorLayer(grassName, fileName, layer, nocats, dataType, outFormat, dsco, lco)
+        exportnocat = self.parameterAsBoolean(parameters, self.GRASS_VECTOR_EXPORT_NOCAT, context)
+        self.exportVectorLayer(grassName, fileName, layer, nocats, dataType, outFormat, dsco, lco, exportnocat)
+
+        self.fileOutputs[name] = fileName
 
     def exportVectorLayer(self, grassName, fileName, layer=None, nocats=False, dataType='auto',
-                          outFormat=None, dsco=None, lco=None):
+                          outFormat=None, dsco=None, lco=None, exportnocat=False):
         """
         Creates a dedicated command to export a vector from
         temporary GRASS DB into a file via OGR.
@@ -919,19 +966,21 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
         :param outFormat: file format for export.
         :param dsco: datasource creation options for format.
         :param lco: layer creation options for format.
+        :param exportnocat: do not export features without categories.
         """
         if outFormat is None:
             outFormat = QgsVectorFileWriter.driverForExtension(os.path.splitext(fileName)[1]).replace(' ', '_')
 
         for cmd in [self.commands, self.outputCommands]:
             cmd.append(
-                'v.out.ogr{0} type="{1}" input="{2}" output="{3}" format="{4}" {5}{6}{7} --overwrite'.format(
-                    '' if nocats else ' -c',
+                'v.out.ogr{0} type="{1}" input="{2}" output="{3}" format="{4}" {5}{6}{7}{8} --overwrite'.format(
+                    '' if nocats else '',
                     dataType, grassName, fileName,
                     outFormat,
                     'layer={}'.format(layer) if layer else '',
                     ' dsco="{}"'.format(dsco) if dsco else '',
-                    ' lco="{}"'.format(lco) if lco else ''
+                    ' lco="{}"'.format(lco) if lco else '',
+                    ' -c' if exportnocat else ''
                 )
             )
 
@@ -981,24 +1030,28 @@ class Grass7Algorithm(QgsProcessingAlgorithm):
     def setSessionProjectionFromProject(self):
         """
         Set the projection from the project.
-        We creates a PROJ4 definition which is transmitted to Grass
+        We create a WKT definition which is transmitted to Grass
         """
         if not Grass7Utils.projectionSet and iface:
-            proj4 = iface.mapCanvas().mapSettings().destinationCrs().toProj4()
-            command = 'g.proj -c proj4="{}"'.format(proj4)
-            self.commands.append(command)
-            Grass7Utils.projectionSet = True
+            self.setSessionProjection(iface.mapCanvas().mapSettings().destinationCrs())
 
     def setSessionProjectionFromLayer(self, layer):
         """
         Set the projection from a QgsVectorLayer.
-        We creates a PROJ4 definition which is transmitted to Grass
+        We create a WKT definition which is transmitted to Grass
         """
         if not Grass7Utils.projectionSet:
-            proj4 = str(layer.crs().toProj4())
-            command = 'g.proj -c proj4="{}"'.format(proj4)
-            self.commands.append(command)
-            Grass7Utils.projectionSet = True
+            self.setSessionProjection(layer.crs())
+
+    def setSessionProjection(self, crs):
+        """
+        Set the session projection to the specified CRS
+        """
+        self.destination_crs = crs
+        file_name = Grass7Utils.exportCrsWktToFile(crs)
+        command = 'g.proj -c wkt="{}"'.format(file_name)
+        self.commands.append(command)
+        Grass7Utils.projectionSet = True
 
     def convertToHtml(self, fileName):
         # Read HTML contents
